# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import mimetypes
from io import BytesIO
from datetime import timedelta, datetime
from unittest.mock import patch

from bson import ObjectId
from eve.utils import ParsedRequest
from nose.tools import assert_raises

from superdesk import get_resource_service, etree
from superdesk.utc import utcnow
from superdesk.errors import SuperdeskApiError, ProviderError
from superdesk.tests import TestCase
from superdesk.tests.setup_teardown import setup_providers, teardown_providers
from superdesk.io import get_feeding_service
from superdesk.io.commands.remove_expired_content import RemoveExpiredContent, get_expired_items
from superdesk.io.feeding_services.file_service import FileFeedingService
from superdesk.io.commands.update_ingest import (
    is_scheduled, update_provider, filter_expired_items, apply_rule_set, get_task_ttl, get_task_id, get_is_idle,
    ingest_item
)


reuters_guid = 'tag_reuters.com_2014_newsml_KBN0FL0NM:10'


def get_file(url, request_kwargs=None):
    """Return file data tuple as returned by media_operations.download_file_from_url"""
    filename = os.path.basename(url)
    dirname = os.path.dirname(os.path.realpath(__file__))
    filepath = os.path.normpath(os.path.join(dirname, 'fixtures', filename))
    mime = mimetypes.guess_type(filepath)[0]
    ext = str(mime).split('/')[1]
    name = str(ObjectId()) + ext
    with open(filepath, 'rb') as f:
        data = f.read()
    return BytesIO(data), name, mime


class UpdateIngestTest(TestCase):

    def setUp(self):
        setup_providers(self)

    def tearDown(self):
        teardown_providers(self)

    def _get_provider(self, provider_name):
        return get_resource_service('ingest_providers').find_one(name=provider_name, req=None)

    def _get_provider_service(self, provider):
        return get_feeding_service(provider['feeding_service'])

    def setup_reuters_provider(self):
        provider_name = 'reuters'
        provider = get_resource_service('ingest_providers').find_one(name=provider_name, req=None)
        provider_service = self._get_provider_service(provider)
        provider_service.provider = provider
        provider_service.URL = provider.get('config', {}).get('url')
        return provider, provider_service

    def test_ingest_items(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        items.extend(provider_service.fetch_ingest(reuters_guid))
        self.assertEqual(12, len(items))
        self.ingest_items(items, provider, provider_service)

    def test_ingest_item_expiry(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        self.assertIsNone(items[1].get('expiry'))
        items[1]['versioncreated'] = utcnow()
        self.ingest_items([items[1]], provider, provider_service)
        self.assertIsNotNone(items[1].get('expiry'))

    def test_ingest_item_sync_if_missing_from_elastic(self):
        provider, provider_service = self.setup_reuters_provider()
        item = provider_service.fetch_ingest(reuters_guid)[0]
        # insert in mongo
        ids = self.app.data._backend('ingest').insert('ingest', [item])
        # check that item is not in elastic
        elastic_item = self.app.data._search_backend('ingest').find_one('ingest', _id=ids[0], req=None)
        self.assertIsNone(elastic_item)
        # trigger sync by fetch
        old_item = get_resource_service('ingest').find_one(_id=ids[0], req=None)
        self.assertIsNotNone(old_item)
        # check that item is synced in elastic
        elastic_item = self.app.data._search_backend('ingest').find_one('ingest', _id=ids[0], req=None)
        self.assertIsNotNone(elastic_item)

    def test_ingest_provider_closed_raises_exception(self):
        provider = {
            'name': 'aap',
            'is_closed': True,
            'source': 'aap',
            'feeding_service': 'file', 'feed_parser': 'nitf',
            'config': {
                'path': '/'
            }
        }

        with assert_raises(SuperdeskApiError) as error_context:
            aap = self._get_provider_service(provider)
            aap.update(provider, {})
        ex = error_context.exception
        self.assertTrue(ex.status_code == 500)

    def test_ingest_provider_closed_when_critical_error_raised(self):
        provider_name = 'AAP'
        provider = self._get_provider(provider_name)
        self.assertFalse(provider.get('is_closed'))
        provider_service = self._get_provider_service(provider)
        provider_service.provider = provider
        provider_service.close_provider(provider, ProviderError.anpaError())
        provider = self._get_provider(provider_name)
        self.assertTrue(provider.get('is_closed'))

    def test_ingest_provider_calls_close_provider(self):
        def mock_update(provider, update):
            raise ProviderError.anpaError()

        provider_name = 'AAP'
        provider = self._get_provider(provider_name)
        self.assertFalse(provider.get('is_closed'))
        provider_service = self._get_provider_service(provider)
        provider_service.provider = provider
        provider_service._update = mock_update
        with assert_raises(ProviderError):
            provider_service.update(provider, {})
        provider = self._get_provider(provider_name)
        self.assertTrue(provider.get('is_closed'))

    def test_is_scheduled(self):
        self.assertTrue(is_scheduled({}), 'run after create')
        self.assertFalse(is_scheduled({'last_updated': utcnow()}), 'wait default time 5m')
        self.assertTrue(is_scheduled({'last_updated': utcnow() - timedelta(minutes=6)}), 'run after 5m')
        self.assertFalse(is_scheduled({
            'last_updated': utcnow() - timedelta(minutes=6),
            'update_schedule': {'minutes': 10}
        }), 'or wait if provider has specific schedule')
        self.assertTrue(is_scheduled({
            'last_updated': utcnow() - timedelta(minutes=11),
            'update_schedule': {'minutes': 10}
        }), 'and run eventually')

    def test_change_last_updated(self):
        ingest_provider = {'name': 'test', 'feeding_service': 'file', 'feed_parser': 'nitf', '_etag': 'test'}
        self.app.data.insert('ingest_providers', [ingest_provider])

        update_provider(ingest_provider)
        provider = self.app.data.find_one('ingest_providers', req=None, _id=ingest_provider['_id'])
        self.assertGreaterEqual(utcnow(), provider.get('last_updated'))
        self.assertEqual('test', provider.get('_etag'))

    def test_filter_expired_items(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        for item in items[:4]:
            item['expiry'] = utcnow() + timedelta(minutes=11)
        self.assertEqual(4, len(filter_expired_items(provider, items)))

    def test_filter_expired_items_with_no_expiry(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        self.assertEqual(0, len(filter_expired_items(provider, items)))

    def test_query_getting_expired_content(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        now = utcnow()
        for i, item in enumerate(items):
            item['ingest_provider'] = provider['_id']
            expiry_time = now - timedelta(hours=11)
            if i > 4:
                expiry_time = now + timedelta(minutes=11)

            item['expiry'] = item['versioncreated'] = expiry_time

        service = get_resource_service('ingest')
        service.post(items)
        expiredItems = get_expired_items(provider, 'ingest')
        self.assertEqual(5, expiredItems.count())

    def test_expiring_with_content(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        now = utcnow()
        for i, item in enumerate(items):
            item['ingest_provider'] = provider['_id']
            expiry_time = now - timedelta(hours=11)
            if i > 4:
                expiry_time = now + timedelta(minutes=11)

            item['expiry'] = item['versioncreated'] = expiry_time

        service = get_resource_service('ingest')
        service.post(items)

        # ingest the items and expire them
        before = service.get(req=None, lookup={})
        self.assertEqual(6, before.count())

        remove = RemoveExpiredContent()
        remove.run(provider.get('type'))

        # only one left in ingest
        after = service.get(req=None, lookup={})
        self.assertEqual(1, after.count())

        req = ParsedRequest()
        self.assertEqual(1, self.app.data.elastic.find('ingest', req, {}).count())
        self.assertEqual(1, self.app.data.mongo.find('ingest', req, {})[1])

    def test_removing_expired_items_from_elastic_only(self):
        now = utcnow()
        self.app.data.elastic.insert('ingest', [
            {'_id': 'foo', 'expiry': now - timedelta(minutes=30)},
            {'_id': 'bar', 'expiry': now + timedelta(minutes=30)},
        ])

        RemoveExpiredContent().run()
        self.assertEqual(1, self.app.data.elastic.find('ingest', ParsedRequest(), {}).count())

    def test_expiring_content_with_files(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        for item in items:
            item['ingest_provider'] = provider['_id']

        now = utcnow()
        items[0]['expiry'] = now - timedelta(hours=11)
        items[1]['expiry'] = now - timedelta(hours=11)
        items[2]['expiry'] = now + timedelta(hours=11)
        items[5]['versioncreated'] = now + timedelta(minutes=11)

        # ingest the items and expire them
        self.ingest_items(items, provider, provider_service)

        # four files in grid fs
        current_files = self.app.media.fs('upload').find()
        self.assertEqual(4, current_files.count())

        remove = RemoveExpiredContent()
        remove.run(provider.get('type'))

        # all gone
        current_files = self.app.media.fs('upload').find()
        self.assertEqual(0, current_files.count())

    def test_apply_rule_set(self):
        item = {'body_html': '@@body@@'}

        provider_name = 'reuters'
        provider = self._get_provider(provider_name)
        self.assertEqual('body', apply_rule_set(item, provider)['body_html'])

        item = {'body_html': '@@body@@'}
        provider_name = 'AAP'
        provider = self._get_provider(provider_name)
        self.assertEqual('@@body@@', apply_rule_set(item, provider)['body_html'])

    def test_all_ingested_items_have_sequence(self):
        provider, provider_service = self.setup_reuters_provider()
        guid = 'tag_reuters.com_2014_newsml_KBN0FL0NM:10'
        item = provider_service.fetch_ingest(guid)[0]
        get_resource_service("ingest").set_ingest_provider_sequence(item, provider)
        self.assertIsNotNone(item['ingest_provider_sequence'])

    def test_get_task_ttl(self):
        self.assertEqual(300, get_task_ttl({}))
        provider = {'update_schedule': {'minutes': 10}}
        self.assertEqual(600, get_task_ttl(provider))
        provider['update_schedule']['hours'] = 1
        provider['update_schedule']['minutes'] = 1
        self.assertEqual(3660, get_task_ttl(provider))

    def test_get_task_id(self):
        provider = {'name': 'foo', '_id': 'abc'}
        self.assertEqual('update-ingest-foo-abc', get_task_id(provider))

    def test_is_idle(self):
        provider = dict(idle_time=dict(hours=1, minutes=0))
        provider['last_item_update'] = utcnow()
        self.assertEqual(get_is_idle(provider), False)
        provider['idle_time']['hours'] = -1
        self.assertEqual(get_is_idle(provider), True)
        provider['idle_time'] = dict(hours=0, minutes=0)
        self.assertEqual(get_is_idle(provider), False)

    def test_files_dont_duplicate_ingest(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)

        for item in items:
            item['ingest_provider'] = provider['_id']
            item['expiry'] = utcnow() + timedelta(hours=11)

        # ingest the items
        self.ingest_items(items, provider, provider_service)

        items = provider_service.fetch_ingest(reuters_guid)
        for item in items:
            item['ingest_provider'] = provider['_id']
            item['expiry'] = utcnow() + timedelta(hours=11)

        # ingest them again
        self.ingest_items(items, provider, provider_service)

        # 12 files in grid fs
        current_files = self.app.media.fs('upload').find()
        self.assertEqual(12, current_files.count())

    def test_anpa_category_to_subject_derived_ingest(self):
        vocab = [{'_id': 'categories', 'items': [{'is_active': True, 'name': 'Domestic Sport', 'qcode': 's',
                                                  "subject": "15000000"}]}]
        self.app.data.insert('vocabularies', vocab)

        provider_name = 'DPA'
        provider = get_resource_service('ingest_providers').find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get('config', {}).get('path', ''), 'IPTC7901_odd_charset.txt')
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        items = [feeding_parser.parse(file_path, provider)]

        # ingest the items and check the subject code has been derived
        self.ingest_items(items, provider, provider_service)
        self.assertEqual(items[0]['subject'][0]['qcode'], '15000000')

    def test_anpa_category_to_subject_derived_ingest_ignores_inactive_categories(self):
        vocab = [{'_id': 'categories', 'items': [{'is_active': False, 'name': 'Domestic Sport', 'qcode': 's',
                                                  "subject": "15000000"}]}]
        self.app.data.insert('vocabularies', vocab)

        provider_name = 'DPA'
        provider = get_resource_service('ingest_providers').find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get('config', {}).get('path', ''), 'IPTC7901_odd_charset.txt')
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        items = [feeding_parser.parse(file_path, provider)]

        # ingest the items and check the subject code has been derived
        self.ingest_items(items, provider, provider_service)
        self.assertNotIn('subject', items[0])

    def test_subject_to_anpa_category_derived_ingest(self):
        vocab = [{'_id': 'iptc_category_map',
                  'items': [{'name': 'Finance', 'category': 'f', 'qcode': '04000000', 'is_active': True}]},
                 {'_id': 'categories',
                  'items': [{'is_active': True, 'name': 'Australian Weather', 'qcode': 'b', 'subject': '17000000'},
                            {'is_active': True, 'name': 'Finance', 'qcode': 'f'}]},
                 {'_id': 'genre', 'items': [{'qcode': 'feature', 'name': 'feature'}]}]

        self.app.data.insert('vocabularies', vocab)

        provider_name = 'AAP'
        provider = get_resource_service('ingest_providers').find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get('config', {}).get('path', ''), 'nitf-fishing.xml')
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        with open(file_path, 'r') as f:
            xml_string = etree.etree.fromstring(f.read())
            items = [feeding_parser.parse(xml_string, provider)]
            for item in items:
                item['ingest_provider'] = provider['_id']
                item['expiry'] = utcnow() + timedelta(hours=11)

            # ingest the items and check the subject code has been derived
            self.ingest_items(items, provider, provider_service)
            self.assertEqual(items[0]['anpa_category'][0]['qcode'], 'f')

    def test_subject_to_anpa_category_derived_ingest_ignores_inactive_map_entries(self):
        vocab = [{'_id': 'iptc_category_map',
                  'items': [{'name': 'Finance', 'category': 'f', 'qcode': '04000000', 'is_active': False}]},
                 {'_id': 'categories',
                  'items': [{'is_active': True, 'name': 'Australian Weather', 'qcode': 'b', 'subject': '17000000'}]},
                 {'_id': 'genre', 'items': [{'qcode': 'feature', 'name': 'feature'}]}]

        self.app.data.insert('vocabularies', vocab)

        provider_name = 'AAP'
        provider = get_resource_service('ingest_providers').find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get('config', {}).get('path', ''), 'nitf-fishing.xml')
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        with open(file_path, 'r') as f:
            xml_string = etree.etree.fromstring(f.read())
            items = [feeding_parser.parse(xml_string, provider)]
            for item in items:
                item['ingest_provider'] = provider['_id']
                item['expiry'] = utcnow() + timedelta(hours=11)

            # ingest the items and check the subject code has been derived
            self.ingest_items(items, provider, provider_service)
            self.assertNotIn('anpa_category', items[0])

    def test_ingest_cancellation(self):
        provider, provider_service = self.setup_reuters_provider()
        guid = 'tag_reuters.com_2016_newsml_L1N14N0FF:978556838'
        items = provider_service.fetch_ingest(guid)
        for item in items:
            item['ingest_provider'] = provider['_id']
            item['expiry'] = utcnow() + timedelta(hours=11)
        self.ingest_items(items, provider, provider_service)
        guid = 'tag_reuters.com_2016_newsml_L1N14N0FF:1542761538'
        items = provider_service.fetch_ingest(guid)
        for item in items:
            item['ingest_provider'] = provider['_id']
            item['expiry'] = utcnow() + timedelta(hours=11)
        self.ingest_items(items, provider, provider_service)
        ingest_service = get_resource_service('ingest')
        lookup = {'uri': items[0].get('uri')}
        family_members = ingest_service.get_from_mongo(req=None, lookup=lookup)
        self.assertEqual(family_members.count(), 2)
        for relative in family_members:
            self.assertEqual(relative['pubstatus'], 'canceled')
            self.assertEqual(relative['state'], 'killed')

    def test_ingest_update(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        items[0]['ingest_provider'] = provider['_id']
        items[0]['expiry'] = utcnow() + timedelta(hours=11)

        self.ingest_items(items, provider, provider_service)

        self.assertEqual(items[0]['unique_id'], 1)
        original_id = items[0]['_id']

        items = provider_service.fetch_ingest(reuters_guid)
        items[0]['ingest_provider'] = provider['_id']
        items[0]['expiry'] = utcnow() + timedelta(hours=11)
        # change the headline
        items[0]['headline'] = 'Updated headline'

        # ingest the item again
        self.ingest_items(items, provider, provider_service)

        # see the update to the headline and unique_id survives
        elastic_item = self.app.data._search_backend('ingest').find_one('ingest', _id=original_id, req=None)
        self.assertEqual(elastic_item['headline'], 'Updated headline')
        self.assertEqual(elastic_item['unique_id'], 1)
        self.assertEqual(elastic_item['unique_name'], '#1')

    def test_get_article_ids(self):
        provider_name = 'reuters'
        provider, provider_service = self.setup_reuters_provider()
        ids = provider_service._get_article_ids('channel1', utcnow(), utcnow() + timedelta(minutes=-10))
        self.assertEqual(len(ids), 3)
        provider = get_resource_service('ingest_providers').find_one(name=provider_name, req=None)
        self.assertEqual(provider['tokens']['poll_tokens']['channel1'], 'ExwaY31kfnR2Z2J1cWZ2YnxoYH9kfw==')

    def test_unknown_category_ingested_is_removed(self):
        vocab = [
            {'_id': 'categories',
             'items': [{'is_active': True, 'name': 'Australian Weather', 'qcode': 'b', 'subject': '17000000'},
                       {'is_active': True, 'name': 'Finance', 'qcode': 'f'}]}
        ]

        self.app.data.insert('vocabularies', vocab)

        provider_name = 'AP'
        provider = get_resource_service('ingest_providers').find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get('config', {}).get('path', ''), 'ap_anpa-3.tst')
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        items = [feeding_parser.parse(file_path, provider)]

        # ingest the items and check the subject code has been derived
        items[0]['versioncreated'] = utcnow()
        self.ingest_items(items, provider, provider_service)
        self.assertTrue(len(items[0]['anpa_category']) == 0)

    def test_ingest_with_routing_keeps_elastic_in_sync(self):
        provider, provider_service = self.setup_reuters_provider()

        desk = {'name': 'foo'}
        self.app.data.insert('desks', [desk])
        self.assertIsNotNone(desk['_id'])
        self.assertIsNotNone(desk['incoming_stage'])

        now = datetime.now()

        items = [
            {
                'guid': 'main_text',
                'versioncreated': now,
                'headline': 'Headline of the text item',
            },
            {'guid': 'image_1', 'type': 'picture', 'versioncreated': now},
            {'guid': 'image_2', 'type': 'picture', 'versioncreated': now},
            {
                'type': 'composite',
                'guid': 'package:guid:abcd123',
                'versioncreated': now,
                'headline': 'Headline of the text item',
                'groups': [
                    {
                        'id': 'root',
                        'role': 'grpRole:NEP',
                        'refs': [{'idRef': 'main'}],
                    }, {
                        'id': 'main',
                        'role': 'main',
                        'refs': [
                            {'residRef': 'main_text'},
                            {'residRef': 'image_1'},
                            {'residRef': 'image_2'},
                        ],
                    }
                ]
            }
        ]

        all_week_schedule = {
            "day_of_week": [
                "MON",
                "TUE",
                "WED",
                "THU",
                "FRI",
                "SAT",
                "SUN"
            ],
            "hour_of_day_from": "00:00:00",
            "hour_of_day_to": "23:55:00",
            "time_zone": "Europe/Prague"
        }

        routing_scheme = {
            "name": "autofetch",
            "rules": [
                {
                    "filter": None,
                    "actions": {
                        "exit": False,
                        "publish": [],
                        "fetch": [
                            {
                                "stage": desk['incoming_stage'],
                                "desk": desk['_id']
                            }
                        ]
                    },
                    "schedule": all_week_schedule,
                    "name": "fetch"
                },
                {
                    "filter": None,
                    "actions": {
                        "exit": False,
                        "publish": [],
                        "fetch": []
                    },
                    "schedule": all_week_schedule,
                    "name": "empty"
                },
            ]
        }

        ingest_service = get_resource_service('ingest')
        self.ingest_items(items, provider, provider_service, routing_scheme=routing_scheme)

        self.assertEqual(4, ingest_service.get_from_mongo(None, {}).count())
        self.assertEqual(4, ingest_service.get(None, {}).count())

        for item in items:
            lookup = {'guid': item['guid']}
            mongo_item = ingest_service.get_from_mongo(None, lookup)[0]
            elastic_item = ingest_service.get(None, lookup)[0]
            self.assertEqual(mongo_item['_etag'], elastic_item['_etag'], mongo_item['guid'])

    @patch('superdesk.media.renditions.download_file_from_url', get_file)
    def test_ingest_associated_item_renditions(self):
        provider = {'feeding_service': 'ninjs', '_id': self.providers['ninjs']}
        provider_service = FileFeedingService()
        item = {
            'guid': 'foo',
            'type': 'text',
            'versioncreated': utcnow(),
            'associations': {
                'featuremedia': {
                    'guid': 'bar',
                    'type': 'picture',
                    'versioncreated': utcnow(),
                    'renditions': {
                        'original': {
                            'href': 'https://farm8.staticflickr.com/7300/picture_bug.jpg',
                            'mimetype': 'image/jpeg',
                            'width': 640,
                            'height': 426,
                        }
                    }
                },
                'foo': {
                    'guid': 'foo',
                    'type': 'picture',
                    'versioncreated': utcnow(),
                    'renditions': {
                        'original': {
                            'href': 'https://farm8.staticflickr.com/7300/picture_bug.jpg',
                            'mimetype': 'image/jpeg',
                            'width': 640,
                            'height': 426,
                        },
                    },
                },
            }
        }

        # avoid transfer_renditions call which would store the picture locally
        # and it would fetch it using superdesk url which doesn't work in test
        with patch('superdesk.io.commands.update_ingest.transfer_renditions'):
            status, ids = ingest_item(item, provider, provider_service)

        self.assertTrue(status)
        self.assertEqual(3, len(ids))
        self.assertIn('thumbnail', item['associations']['featuremedia']['renditions'])
        self.assertIn('thumbnail', item['associations']['foo']['renditions'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import hashlib
from superdesk.tests import TestCase
from superdesk.io.feed_parsers.bbc_ninjs import BBCNINJSFeedParser
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_TAG
from superdesk.metadata.utils import generate_guid


class BBCNINJSTestCase(TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}

        with open(fixture, 'r') as json_file:
            data = json_file.read()

        self.items = BBCNINJSFeedParser().parse(data, provider)


class SimpleTestCase(BBCNINJSTestCase):

    filename = 'bbc-ninjs-text-test.json'

    def test_trans_attributes(self):
        self.assertEqual(self.items[0].get(ITEM_TYPE), CONTENT_TYPE.TEXT)
        self.assertEqual(self.items[0].get('subject')[0].get('qcode'), '11016007')

        guid_hash = hashlib.sha1('https://www.example.com//12345'.encode('utf8')).hexdigest()
        guid = generate_guid(type=GUID_TAG, id=guid_hash)
        self.assertEqual(self.items[0].get('guid'), guid)


class CompositeTestCase(BBCNINJSTestCase):

    filename = 'bbc-ninjs-comp-test.json'

    def test_parsed_items(self):
        # The picture
        self.assertEqual(self.items[1].get(ITEM_TYPE), CONTENT_TYPE.PICTURE)
        self.assertEqual(self.items[1].get('headline'), 'logo-footer.png')
        self.assertEqual(self.items[1].get('description_text'), 'abc')

        # The text item
        self.assertEqual(self.items[0].get(ITEM_TYPE), CONTENT_TYPE.TEXT)
        self.assertEqual(self.items[0].get('headline'), 'abcdef')

        # The associated picture in the text item
        self.assertEqual(self.items[0].get('associations').get('featuremedia').get(ITEM_TYPE), CONTENT_TYPE.PICTURE)
        self.assertEqual(self.items[0].get('associations').get('featuremedia').get('headline'), 'logo-footer.png')
        self.assertEqual(self.items[0].get('associations').get('featuremedia').get('description_text'), 'abc')

        # The composite
        self.assertEqual(self.items[2].get(ITEM_TYPE), CONTENT_TYPE.COMPOSITE)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Superdesk"""

import blinker
import logging as logging_lib
from flask import abort, json, Blueprint, current_app as app  # noqa
from flask_script import Command as BaseCommand, Option  # noqa
from werkzeug.exceptions import HTTPException
from eve.utils import config  # noqa
from eve.methods.common import document_link  # noqa

from .eve_backend import EveBackend
from .datalayer import SuperdeskDataLayer  # noqa
from .services import BaseService as Service  # noqa
from .resource import Resource  # noqa
from .privilege import privilege, intrinsic_privilege, get_intrinsic_privileges  # noqa
from .workflow import *  # noqa
from .signals import *  # noqa

__version__ = '2.0.8'

API_NAME = 'Superdesk API'
SCHEMA_VERSION = 0
DOMAIN = {}
COMMANDS = {}
JINJA_FILTERS = dict()
app_components = dict()
app_models = dict()
resources = dict()
_eve_backend = EveBackend()
default_user_preferences = dict()
default_session_preferences = dict()
logger = logging_lib.getLogger(__name__)


class Command(BaseCommand):
    """Superdesk Command.

    The Eve framework changes introduced with https://github.com/nicolaiarocci/eve/issues/213 make the commands fail.
    Reason being the flask-script's run the commands using test_request_context() which is invalid.
    That's the reason we are inheriting the Flask-Script's Command to overcome this issue.
    """

    def __call__(self, _app=None, *args, **kwargs):
        try:
            with app.app_context():
                res = self.run(*args, **kwargs)
                logger.info('Command finished with: {}'.format(res))
                return 0
        except Exception as ex:
            logger.info('Uhoh, an exception occured while running the command...')
            logger.exception(ex)
            return 1


def get_headers(self, environ=None):
    """Fix CORS for abort responses.

    todo(petr): put in in custom flask error handler instead
    """
    return [
        ('Content-Type', 'text/html'),
        ('Access-Control-Allow-Origin', '*'),
        ('Access-Control-Allow-Headers', '*'),
    ]


setattr(HTTPException, 'get_headers', get_headers)


def domain(resource, res_config):
    """Register domain resource"""
    DOMAIN[resource] = res_config


def command(name, command):
    """Register command"""
    COMMANDS[name] = command


def blueprint(blueprint, app, **kwargs):
    """Register flask blueprint.

    :param blueprint: blueprint instance
    :param app: flask app instance
    """
    blueprint.kwargs = kwargs
    prefix = app.api_prefix or None
    app.register_blueprint(blueprint, url_prefix=prefix, **kwargs)


def get_backend():
    """Returns the available backend, this will be changed in a factory if needed."""
    return _eve_backend


def get_resource_service(resource_name):
    return resources[resource_name].service


def get_resource_privileges(resource_name):
    attr = getattr(resources[resource_name], 'privileges', {})
    return attr


def register_default_user_preference(preference_name, preference):
    default_user_preferences[preference_name] = preference


def register_default_session_preference(preference_name, preference):
    default_session_preferences[preference_name] = preference


def register_resource(name, resource, service=None, backend=None, privilege=None, _app=None):
    """Shortcut for registering resource and service together.

    :param name: resource name
    :param resource: resource class
    :param service: service class
    :param backend: backend instance
    :param privilege: privilege to register with resource
    :param _app: flask app
    """
    if not backend:
        backend = get_backend()
    if not service:
        service = Service
    if privilege:
        intrinsic_privilege(name, privilege)
    if not _app:
        _app = app
    service_instance = service(name, backend=backend)
    resource(name, app=_app, service=service_instance)


def register_jinja_filter(name, jinja_filter):
    """Register jinja filter

    :param str name: name of the filter
    :param jinja_filter: jinja filter function
    """
    JINJA_FILTERS[name] = jinja_filter


def register_item_schema_field(name, schema, app, copy_on_rewrite=True):
    """Register new item schema field.

    .. versionadded:: 1.28

    :param str name: field name
    :param dict schema: field schema
    :param Flask app: flask app
    :param bool copy_on_rewrite: copy field value when rewriting item
    """
    for resource in ['ingest', 'archive', 'published', 'archive_autosave']:
        app.config['DOMAIN'][resource]['schema'].update({name: schema})
        app.config['DOMAIN'][resource]['datasource']['projection'].update({name: 1})

    app.config['DOMAIN']['content_templates_apply']['schema']['item']['schema'].update(
        {name: schema}
    )

    if copy_on_rewrite:
        app.config.setdefault('COPY_ON_REWRITE_FIELDS', [])
        app.config['COPY_ON_REWRITE_FIELDS'].append(name)


from superdesk.search_provider import SearchProvider  # noqa
from apps.search_providers import register_search_provider  # noqa



import os
import json
import pytest
from pathlib import Path
from bson import ObjectId
from flask import url_for
from requests.auth import _basic_auth_str

from superdesk.tests import get_mongo_uri, setup, clean_dbs
from superdesk.factory import get_app as get_sd_app
from superdesk.auth_server.clients import RegisterClient
from prod_api.app import get_app as get_prodapi_api


MONGO_DB = 'prodapi_tests'
ELASTICSEARCH_INDEX = MONGO_DB
AUTH_SERVER_SHARED_SECRET = '2kZOf0VI9T70vU9uMlKLyc5GlabxVgl6'


def get_test_prodapi_app(extra_config=None):
    """
    Create and return configured test prod api flask app.
    :param extra_config: extra settings
    :return: eve.flaskapp.Eve
    """
    test_config = {
        'DEBUG': True,
        'TESTING': True,
        'SUPERDESK_TESTING': True,
        'MONGO_CONNECT': False,
        'MONGO_MAX_POOL_SIZE': 1,
        'MONGO_DBNAME': MONGO_DB,
        'MONGO_URI': get_mongo_uri('MONGO_URI', MONGO_DB),
        'ELASTICSEARCH_INDEX': ELASTICSEARCH_INDEX,
        'PRODAPI_URL': 'http://localhost:5500',
        'MEDIA_PREFIX': 'http://localhost:5500/prodapi/v1/assets',
        'PRODAPI_URL_PREFIX': 'prodapi',
        'URL_PREFIX': 'prodapi',
        'AUTH_SERVER_SHARED_SECRET': AUTH_SERVER_SHARED_SECRET
    }
    if extra_config:
        test_config.update(extra_config)
    prodapi_app = get_prodapi_api(test_config)

    # put elastic mapping
    with prodapi_app.app_context():
        prodapi_app.data.elastic.init_index()

    return prodapi_app


def get_test_superdesk_app(extra_config=None):
    """
    Create and return configured test superdesk flask app.
    :param extra_config: extra settings
    :return: eve.flaskapp.Eve
    """
    test_config = {
        'MONGO_DBNAME': MONGO_DB,
        'MONGO_URI': get_mongo_uri('MONGO_URI', MONGO_DB),
        'ELASTICSEARCH_INDEX': ELASTICSEARCH_INDEX,
        'AUTH_SERVER_SHARED_SECRET': AUTH_SERVER_SHARED_SECRET,
    }
    if extra_config:
        test_config.update(extra_config)

    def context():
        pass

    context.app = None
    context.ctx = None
    context.client = None
    setup(context=context, config=test_config, app_factory=get_sd_app)

    return context.app


def teardown_app(app):
    """
    Drop test db and test app
    """
    clean_dbs(app)
    del app


@pytest.fixture(scope='function')
def superdesk_app(request):
    """
    Superdesk app.

    :return: superdesk app
    :rtype: superdesk.factory.app.SuperdeskEve
    """

    extra_config = getattr(request, 'param', {})
    app = get_test_superdesk_app(extra_config)

    def test_app_teardown():
        """
        Drop test db and test app
        """
        teardown_app(app)

    request.addfinalizer(test_app_teardown)

    return app


@pytest.fixture(scope='function')
def prodapi_app(request):
    """
    Prod api app.

    :return: prod api app
    :rtype: eve.flaskapp.Eve
    """

    extra_config = getattr(request, 'param', {})
    app = get_test_prodapi_app(extra_config)

    def test_app_teardown():
        """
        Drop test db and test app
        """
        teardown_app(app)

    request.addfinalizer(test_app_teardown)

    return app


@pytest.fixture(scope='module')
def prodapi_app_with_data(request):
    """
    Prod api app with prefilled collections and with disabled auth.
    ATTENTION: This is a resource-heavy fixture and it's designed to use with "module" scope.
    It's better to use it in readonly tests and not modify data of fixtured app.

    :return: prod api app
    :rtype: eve.flaskapp.Eve
    """

    extra_config = getattr(request, 'param', {})
    extra_config['PRODAPI_AUTH_ENABLED'] = False
    app = get_test_prodapi_app(extra_config)

    # fill with data
    with app.app_context():
        p = Path(os.path.join(os.path.dirname(__file__), 'tests/fixtures'))
        for fixture_file in [x for x in p.iterdir() if x.is_file()]:
            with fixture_file.open() as f:
                app.data.insert(
                    resource=fixture_file.stem,
                    docs=json.loads(f.read())
                )

    def test_app_teardown():
        """
        Drop test db and test app
        """
        teardown_app(app)

    request.addfinalizer(test_app_teardown)

    return app


@pytest.fixture(scope='module')
def prodapi_app_with_data_client(prodapi_app_with_data):
    """Test client for prod api with filled data"""

    client = prodapi_app_with_data.test_client()

    with prodapi_app_with_data.app_context():
        yield client


@pytest.fixture(scope='function')
def prodapi_client(prodapi_app):
    """Test client for prod api"""

    client = prodapi_app.test_client()

    with prodapi_app.app_context():
        yield client


@pytest.fixture(scope='function')
def superdesk_client(superdesk_app):
    """Test client for superdesk"""

    client = superdesk_app.test_client()

    with superdesk_app.app_context():
        yield client


@pytest.fixture(scope='function')
def auth_server_registered_clients(request, superdesk_app):
    """
    Registers clients for auth server.
    :return: dict with clients
    """
    clients_data = []

    with superdesk_app.app_context():
        for param in request.param:
            # register clients
            clients_data.append({
                "name": str(ObjectId()),  # just a random string
                "client_id": str(ObjectId()),
                "password": str(ObjectId()),  # just a random string
                "scope": param
            })
            RegisterClient().run(**clients_data[-1])

    return clients_data


@pytest.fixture(scope='function')
def issued_tokens(request, superdesk_app, superdesk_client):
    tokens = []
    clients_data = []

    # register clients
    with superdesk_app.app_context():
        for param in request.param:
            clients_data.append({
                "name": str(ObjectId()),  # just a random string
                "client_id": str(ObjectId()),
                "password": str(ObjectId()),  # just a random string
                "scope": param
            })
            RegisterClient().run(**clients_data[-1])

    # retrieve tokens
    with superdesk_app.test_request_context():
        for client_data in clients_data:
            resp = superdesk_client.post(
                url_for('auth_server.issue_token'),
                data={
                    'grant_type': 'client_credentials'
                },
                headers={
                    'Authorization': _basic_auth_str(client_data['client_id'], client_data['password'])
                }
            )
            tokens.append(json.loads(resp.data.decode('utf-8')))

    teardown_app(superdesk_app)
    del superdesk_client

    return tokens



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import copy
import glob
import os
import shutil
import tempfile
from unittest import mock
import datetime
import pytz

from superdesk.tests import setup
from superdesk.tests import TestCase as CoreTestCase
from superdesk.io.feeding_services import ftp
from superdesk.utc import utcnow, utc

PREFIX = 'test_superdesk_'
PROVIDER = {
    "_id": "test_provider",
    "name": "Test provider",
    "config": {
        "passive": True,
        "username": "user",
        "password": "password",
        "host": "example.net",
        "dest_path": "/tmp",
        "path": "",
        "move": True,
        "ftp_move_path": "dest_move",
        "move_path_error": "error",
        "field_aliases": []
    },
    "last_updated": datetime.datetime(2017, 5, 16, 16, 47, 39, tzinfo=pytz.utc),
    "last_item_update": datetime.datetime(2017, 5, 16, 16, 47, 39, tzinfo=pytz.utc)
}


def ftp_file(filename, modify):
    facts = mock.Mock()
    facts.get.return_value = 'file'
    facts.__getitem__ = mock.Mock(return_value=modify)
    return [filename, facts]


def ingest_items(generator, ingest_status=True):
    failed = None
    while True:
        try:
            item = generator.send(failed)
            failed = set([item['guid']]) if not ingest_status else set()
        except StopIteration:
            break


class FakeFTP(mock.MagicMock):

    files = [
        ftp_file('filename_1.xml', '20170517164739'),
        ftp_file('filename_2.xml', '20170517164739'),
        ftp_file('filename_3.xml', '20170517164739'),
        ftp_file('filename_4.xml', '20170517164739'),
        ftp_file('filename_5.xml', '20170517164745'),
        ftp_file('filename_6.xml', '20170517164745'),
        ftp_file('filename_7.xml', '20170517164745'),
        ftp_file('filename_8.xml', '20170517164745'),
        ftp_file('filename_9.xml', '20170517164746'),
        ftp_file('filename_10.xml', '20170517164748'),
        ftp_file('filename_11.xml', '20170517164748'),
        ftp_file('filename_12.xml', '20170517164748'),
        ftp_file('filename_13.xml', '20170517164748'),
        ftp_file('filename_14.xml', '20170517164748'),
        ftp_file('filename_15.xml', '20170517164755'),
        ftp_file('filename_16.xml', '20170517164756')
    ]

    def mlsd(self, path=""):
        return iter(self.files)

    def cwd(self, path):
        pass


class FakeFTPRecentFiles(FakeFTP):

    files = [
        ftp_file('old_file.xml', '20170517164756'),
        # we need a file ingested now, before INGEST_OLD_CONTENT_MINUTES is expired
        ftp_file('recent_file.xml', datetime.datetime.today().strftime('%Y%m%d%H%M%S')),
    ]


class FakeFeedParser(mock.MagicMock):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        del self.ALLOWED_EXT


class FailingFakeFeedParser(FakeFeedParser):

    def parse(self, *args, **kwargs):
        raise Exception('Test exception')


class TestCase(CoreTestCase):

    def setUpForChildren(self):
        """Run this `setUp` stuff for each children.

        Configure new `app` for each test.
        """
        setup.reset = True
        setup(self)

        self.ctx = self.app.app_context()
        self.ctx.push()

        def clean_ctx():
            if self.ctx:
                self.ctx.pop()

        self.addCleanup(clean_ctx)


class FTPTestCase(TestCase):

    def test_it_can_connect(self):
        service = ftp.FTPFeedingService()

        if 'FTP_URL' not in os.environ:
            return

        config = service.config_from_url(os.environ['FTP_URL'])
        self.assertEqual('test', config['path'])
        self.assertEqual('localhost', config['host'])

        config['dest_path'] = tempfile.mkdtemp(prefix=PREFIX)
        provider = {'config': config}

        items = service.update(provider, {})
        self.assertEqual(266, len(items))

        provider['last_updated'] = utcnow()
        self.assertEqual(0, len(service.update(provider, {})))

        self.assertTrue(os.path.isdir(provider['config']['dest_path']))
        self.assertEqual(266, len(os.listdir(provider['config']['dest_path'])))

    def tearDown(self):
        for folder in glob.glob('/tmp/%s*' % (PREFIX)):
            shutil.rmtree(folder)

    @mock.patch.object(os.path, 'getsize', return_value=3)
    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_ingested(self, ftp_connect, *args):
        """Check that ingested file is moved if "move" is set

        feature requested in SDESK-468
        """
        provider = copy.deepcopy(PROVIDER)
        service = ftp.FTPFeedingService()
        service._is_empty = mock.MagicMock(return_value=False)
        ingest_items(service.update(provider, {}))

        mock_ftp = ftp_connect.return_value.__enter__.return_value

        self.assertEqual(mock_ftp.rename.call_count, len(FakeFTP.files))
        for i, call in enumerate(mock_ftp.rename.call_args_list):
            self.assertEqual(
                call[0],
                (FakeFTP.files[i][0], 'dest_move/{}'.format(FakeFTP.files[i][0]))
            )

    @mock.patch.object(os.path, 'getsize', return_value=3)
    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_ingested_default(self, ftp_connect, *args):
        """Check that ingested file is moved to default path if "move" is empty string

        feature requested in SDESK-1452
        """
        provider = copy.deepcopy(PROVIDER)
        provider['config']['ftp_move_path'] = ""
        service = ftp.FTPFeedingService()
        service._is_empty = mock.MagicMock(return_value=False)
        ingest_items(service.update(provider, {}))
        mock_ftp = ftp_connect.return_value.__enter__.return_value

        self.assertEqual(mock_ftp.rename.call_count, len(FakeFTP.files))
        for i, call in enumerate(mock_ftp.rename.call_args_list):
            self.assertEqual(
                call[0],
                (FakeFTP.files[i][0], '{}/{}'.format(ftp.DEFAULT_SUCCESS_PATH, FakeFTP.files[i][0]))
            )

    @mock.patch.object(os.path, 'getsize', return_value=3)
    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_ingested_error(self, ftp_connect, *args):
        """Check that ingested file is moved to error path if ingest fails"""
        provider = copy.deepcopy(PROVIDER)
        provider['config']['ftp_move_path'] = ""
        service = ftp.FTPFeedingService()
        service._is_empty = mock.MagicMock(return_value=False)
        ingest_items(service.update(provider, {}), False)
        mock_ftp = ftp_connect.return_value.__enter__.return_value

        self.assertEqual(mock_ftp.rename.call_count, len(FakeFTP.files))
        for i, call in enumerate(mock_ftp.rename.call_args_list):
            self.assertEqual(
                call[0],
                (FakeFTP.files[i][0], '{}/{}'.format('error', FakeFTP.files[i][0]))
            )

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_ingested_no_move(self, ftp_connect):
        """Check that ingested file is not moved if "move" is not set

        feature requested in SDESK-468
        """
        provider = copy.deepcopy(PROVIDER)
        provider['config']['move'] = False
        service = ftp.FTPFeedingService()
        ingest_items(service.update(provider, {}))
        mock_ftp = ftp_connect.return_value.__enter__.return_value
        mock_ftp.rename.assert_not_called()

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTPRecentFiles)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_backstop(self, ftp_connect):
        """Check that failing file is not moved if it's more recent thant INGEST_OLD_CONTENT_MINUTES"""
        provider = copy.deepcopy(PROVIDER)
        service = ftp.FTPFeedingService()
        service._is_empty = mock.MagicMock(return_value=False)
        ingest_items(service.update(provider, {}))
        mock_ftp = ftp_connect.return_value.__enter__.return_value

        # recent_file must not have been ingested
        self.assertEqual(mock_ftp.rename.call_count, len(FakeFTPRecentFiles.files) - 1)
        for i, call in enumerate(mock_ftp.rename.call_args_list):
            self.assertNotEqual(
                call[0][0],
                'recent_file.xml',
            )

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FailingFakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_error(self, ftp_connect):
        """Check that error on ingestion moves item if "move_path_error" is set

        feature requested in SDESK-1452
        """
        provider = copy.deepcopy(PROVIDER)
        service = ftp.FTPFeedingService()
        service._is_empty = mock.MagicMock(return_value=False)
        ingest_items(service.update(provider, {}))
        mock_ftp = ftp_connect.return_value.__enter__.return_value

        self.assertEqual(mock_ftp.rename.call_count, len(FakeFTP.files))
        for i, call in enumerate(mock_ftp.rename.call_args_list):
            self.assertEqual(
                call[0],
                (FakeFTP.files[i][0], 'error/{}'.format(FakeFTP.files[i][0]))
            )

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FailingFakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_error_default(self, ftp_connect):
        """Check that error on ingestion use default path if "move_path_error" is empty string

        feature requested in SDESK-1452
        """
        provider = copy.deepcopy(PROVIDER)
        provider['config']['move_path_error'] = ""
        service = ftp.FTPFeedingService()
        service._is_empty = mock.MagicMock(return_value=False)
        ingest_items(service.update(provider, {}))
        mock_ftp = ftp_connect.return_value.__enter__.return_value

        self.assertEqual(mock_ftp.rename.call_count, len(FakeFTP.files))
        for i, call in enumerate(mock_ftp.rename.call_args_list):
            self.assertEqual(
                call[0],
                (FakeFTP.files[i][0], '{}/{}'.format(ftp.DEFAULT_FAILURE_PATH, FakeFTP.files[i][0]))
            )

    def test_allowed_suffix_json(self):
        """Check that json files are allowed for ingestion."""
        service = ftp.FTPFeedingService()
        allowed = ftp.FTPFeedingService.ALLOWED_EXT_DEFAULT
        self.assertFalse(service._is_allowed('foo.jpg', allowed))
        self.assertTrue(service._is_allowed('foo.xml', allowed))
        self.assertTrue(service._is_allowed('foo.json', allowed))
        self.assertTrue(service._is_allowed('foo.JSON', allowed))
        self.assertFalse(service._is_allowed('foojson', allowed))
        self.assertFalse(service._is_allowed('foo.json.tar.gz', allowed))

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch.object(ftp.FTPFeedingService, '_retrieve_and_parse')
    def test_files_limit_no_move(self, *mocks):
        """Test file limits when move is off

        feature requested in SDESK-3815
        """
        update = {}
        self.app.config['FTP_INGEST_FILES_LIST_LIMIT'] = 3

        retrieve_and_parse, ftp_connect = mocks
        provider = copy.deepcopy(PROVIDER)
        provider['config']['move'] = False
        service = ftp.FTPFeedingService()
        service._is_empty = mock.MagicMock(return_value=False)
        mock_ftp = ftp_connect.return_value.__enter__.return_value

        ingest_items(service.update(provider, update))
        provider.update(update)
        self.assertEqual(retrieve_and_parse.call_count, 3)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164739', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        self.assertEqual(retrieve_and_parse.call_count, 8)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164745', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        self.assertEqual(retrieve_and_parse.call_count, 13)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164746', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        self.assertEqual(retrieve_and_parse.call_count, 16)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164748', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        self.assertEqual(retrieve_and_parse.call_count, 22)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164755', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        self.assertEqual(retrieve_and_parse.call_count, 24)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164756', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        self.assertEqual(mock_ftp.rename.call_count, 0)

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch.object(ftp.FTPFeedingService, '_retrieve_and_parse')
    def test_files_limit_move(self, *mocks):
        """Test file limits when move is on

        feature requested in SDESK-3815
        """
        update = {}
        self.app.config['FTP_INGEST_FILES_LIST_LIMIT'] = 3

        retrieve_and_parse, ftp_connect = mocks
        provider = copy.deepcopy(PROVIDER)
        service = ftp.FTPFeedingService()
        service._is_empty = mock.MagicMock(return_value=False)
        mock_ftp = ftp_connect.return_value.__enter__.return_value

        ingest_items(service.update(provider, update))
        provider.update(update)
        # emulate moving files by reducing list
        ftp_connect().__enter__().mlsd = mock.Mock()
        ftp_connect().__enter__().mlsd.return_value = iter(FakeFTP.files[3:])

        self.assertEqual(retrieve_and_parse.call_count, 3)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164739', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        # emulate moving files by reducing list
        ftp_connect().__enter__().mlsd = mock.Mock()
        ftp_connect().__enter__().mlsd.return_value = iter(FakeFTP.files[6:])

        self.assertEqual(retrieve_and_parse.call_count, 6)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164745', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        # emulate moving files by reducing list
        ftp_connect().__enter__().mlsd = mock.Mock()
        ftp_connect().__enter__().mlsd.return_value = iter(FakeFTP.files[9:])

        self.assertEqual(retrieve_and_parse.call_count, 9)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164746', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        # emulate moving files by reducing list
        ftp_connect().__enter__().mlsd = mock.Mock()
        ftp_connect().__enter__().mlsd.return_value = iter(FakeFTP.files[12:])

        self.assertEqual(retrieve_and_parse.call_count, 12)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164748', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        # emulate moving files by reducing list
        ftp_connect().__enter__().mlsd = mock.Mock()
        ftp_connect().__enter__().mlsd.return_value = iter(FakeFTP.files[15:])

        self.assertEqual(retrieve_and_parse.call_count, 15)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164755', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        ingest_items(service.update(provider, update))
        provider.update(update)
        # emulate moving files by reducing list
        ftp_connect().__enter__().mlsd = mock.Mock()
        ftp_connect().__enter__().mlsd.return_value = iter(FakeFTP.files[16:])

        self.assertEqual(retrieve_and_parse.call_count, 16)
        self.assertEqual(
            provider['private']['last_processed_file_modify'],
            datetime.datetime.strptime('20170517164756', '%Y%m%d%H%M%S').replace(tzinfo=utc)
        )

        self.assertEqual(mock_ftp.rename.call_count, 16)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from flask import request


class WorkspaceResource(superdesk.Resource):

    schema = {
        'name': {'type': 'string', 'unique_to_user': True},
        'widgets': {'type': 'list'},
        'desk': superdesk.Resource.rel('desks'),
        'user': superdesk.Resource.rel('users'),
        'init_version': {'type': 'integer'},
    }
    item_methods = ['GET', 'PATCH', 'DELETE']


class WorkspaceService(superdesk.Service):

    def is_authorized(self, **kwargs):
        if kwargs.get('_id'):
            data = self.find_one(req=None, _id=kwargs.get('_id')) or request.get_json()
        else:
            data = request.get_json()
        # TODO(petr): use privileges to test who can save desk/role dashboard
        return 'user' not in data or str(data['user']) == str(kwargs['user_id'])




import bson
import logging
import superdesk

from superdesk.utc import utcnow
from superdesk.metadata.item import ITEM_STATE, PUBLISH_STATES


logger = logging.getLogger(__name__)


def track_usage(media_item, stored_item, item_obj, item_name, original):
    if not media_item:
        return

    try:
        orig_id = original['associations'][item_name]['_id']
    except (AttributeError, TypeError, KeyError):
        orig_id = None

    if item_obj['_id'] != orig_id:
        _update_usage(media_item)
        stored_item['used'] = True


def _update_usage(item):
    updates = {
        'used': True,
        'used_count': item.get('used_count', 0) + 1,
        'used_updated': utcnow(),
    }

    superdesk.get_resource_service('archive').system_update(item['_id'], updates, item)

    # update published item state as well
    if item.get(ITEM_STATE) in PUBLISH_STATES:
        published = superdesk.get_resource_service('published').get_last_published_version(item['_id'])
        if published:
            superdesk.get_resource_service('published').system_update(bson.ObjectId(published['_id']),
                                                                      updates, published)
        else:
            logger.warning('published item not found for item %s', item['_id'])

    item.update(updates)


def update_refs(updates, original):
    """Update refs stored on item based on its associations.

    We can't use associations for queries due to unknown keys in dict,
    so storing basic metadata again as list in `refs`.
    """
    if 'associations' not in updates:
        return
    refs = []
    assoc = original['associations'].copy() if original.get('associations') else {}
    assoc.update(updates['associations'] or {})
    for key, val in assoc.items():
        if not val:
            continue
        if val.get('_id') and not val.get('guid'):
            # for related items we only store the _id, fetch other metadata
            item = superdesk.get_resource_service('archive').find_one(req=None, _id=val['_id']) or {}
        else:
            item = {}
        refs.append({
            'key': key,
            '_id': val.get('_id'),
            'uri': val.get('uri') or item.get('uri'),
            'guid': val.get('guid') or item.get('guid'),
            'type': val.get('type') or item.get('type'),
        })
    updates['refs'] = refs




from flask import current_app as app
from superdesk import get_resource_service


def add_missing_keywords(sender, item, **kwargs):
    if app.config.get('KEYWORDS_ADD_MISSING_ON_PUBLISH') and item.get('keywords'):
        get_resource_service('vocabularies').add_missing_keywords(item['keywords'], item.get('language'))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime

import superdesk
from superdesk.tests import TestCase


class ArchiveGetItemsChainTestCase(TestCase):
    archive = [
        {
            '_id': 'original',
            'guid': 'original',
            'headline': 'Original EN',
            "translations": [
                "original-translation-fr",
                "original-translation-nl"
            ],
            "rewritten_by": "update-1",
            'language': 'en',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
        {
            '_id': 'original-translation-fr',
            'guid': 'original-translation-fr',
            'headline': 'Original Translation FR',
            "translated_from": "original",
            'language': 'fr',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
        {
            '_id': 'original-translation-nl',
            'guid': 'original-translation-nl',
            'headline': 'Original Translation NL',
            "translated_from": "original",
            'language': 'nl',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
        {
            '_id': 'update-1',
            'guid': 'update-1',
            'headline': 'Update 1 EN',
            "translations": [],
            "rewritten_by": "update-2",
            "rewrite_of": "original",
            'language': 'en',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
        {
            '_id': 'update-2',
            'guid': 'update-2',
            'headline': 'Update 2 EN',
            "translations": [
                "update-2-translation-fr",
            ],
            "rewrite_of": "update-1",
            'language': 'en',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
        {
            '_id': 'update-2-translation-fr',
            'guid': 'update-2-translation-fr',
            'headline': 'Update 2 Translation FR',
            "translated_from": "update-2",
            "translations": [
                "update-2-translation-fr-nl",
            ],
            'language': 'fr',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
        {
            '_id': 'update-2-translation-fr-nl',
            'guid': 'update-2-translation-fr-nl',
            'headline': 'Update 2 Translation FR NL',
            "translated_from": "update-2-translation-fr",
            'language': 'nl',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
        {
            '_id': 'update-2-translation-fr-nl-update',
            'guid': 'update-2-translation-fr-nl-update',
            'headline': 'Update 2 Translation FR NL update',
            "rewrite_of": "update-2-translation-fr-nl",
            'language': 'nl',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
        {
            '_id': 'update-2-translation-fr-nl-update-2',
            'guid': 'update-2-translation-fr-nl-update-2',
            'headline': 'Update 2 Translation FR NL update',
            "rewrite_of": "update-2-translation-fr-nl-update",
            'language': 'nl',
            'type': 'text',
            'version': 1,
            'profile': 'text',
            'pubstatus': 'usable',
            'format': 'HTML',
            'firstcreated': datetime.datetime(2019, 4, 3, 12, 41, 53),
            'versioncreated': datetime.datetime(2019, 4, 3, 12, 45, 14),
            'original_creator': '5d385f31fe985ec67a0ca583',
            'state': 'published',
            'source': 'Belga',
            'version_creator': '5d385f31fe985ec67a0ca583',
            'slugline': 'skoda scala',
            'byline': 'BELGA',
        },
    ]

    def setUp(self):
        self.app.data.insert('archive', self.archive)

    def test_get_items_chain(self):
        archive_service = superdesk.get_resource_service('archive')

        original = self.archive[0]
        items = archive_service.get_items_chain(original)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl']
        )

        original_translation_fr = self.archive[1]
        items = archive_service.get_items_chain(original_translation_fr)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl']
        )

        original_translation_nl = self.archive[2]
        items = archive_service.get_items_chain(original_translation_nl)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl']
        )

        update_1 = self.archive[3]
        items = archive_service.get_items_chain(update_1)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl', 'update-1']
        )

        update_2 = self.archive[4]
        items = archive_service.get_items_chain(update_2)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl',
             'update-1', 'update-2', 'update-2-translation-fr', 'update-2-translation-fr-nl']
        )

        update_2_translation_fr = self.archive[5]
        items = archive_service.get_items_chain(update_2_translation_fr)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl',
             'update-1', 'update-2', 'update-2-translation-fr', 'update-2-translation-fr-nl']
        )

        update_2_translation_fr_nl = self.archive[6]
        items = archive_service.get_items_chain(update_2_translation_fr_nl)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl',
             'update-1', 'update-2', 'update-2-translation-fr', 'update-2-translation-fr-nl']
        )

        update_2_translation_fr_nl_update = self.archive[7]
        items = archive_service.get_items_chain(update_2_translation_fr_nl_update)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl',
             'update-1', 'update-2', 'update-2-translation-fr', 'update-2-translation-fr-nl',
             'update-2-translation-fr-nl-update']
        )

        update_2_translation_fr_nl_update_2 = self.archive[8]
        items = archive_service.get_items_chain(update_2_translation_fr_nl_update_2)
        self.assertListEqual(
            [i['_id'] for i in items],
            ['original', 'original-translation-fr', 'original-translation-nl',
             'update-1', 'update-2', 'update-2-translation-fr', 'update-2-translation-fr-nl',
             'update-2-translation-fr-nl-update', 'update-2-translation-fr-nl-update-2']
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .volume_cubic_inches_to_metric import cubic_inches_to_metric


class VolumeTestCase(unittest.TestCase):

    def test(self):
        text = 'Total volume is 100.50 cubic inches for this land. '\
               'Total volume is 15.7 cubic in for this land. '\
               'Total volume is 1 Cubic Inch for this land. '\
               'Total volume is 1-16 cu-in for this land. '\
               'Total volume is 1-16 cb. in for this land. '\
               'Total volume is 16.7-Cubic-in for this land. '\
               'Total volume is 16,500-cu. in. for this land. '\

        item = {'body_html': text}
        res, diff = cubic_inches_to_metric(item)
        self.assertEqual(diff['100.50 cubic inches'], '100.50 cubic inches (1,647 cubic centimeter)')
        self.assertEqual(diff['15.7 cubic in'], '15.7 cubic in (257.3 cubic centimeter)')
        self.assertEqual(diff['1 Cubic Inch'], '1 Cubic Inch (16 cubic centimeter)')
        self.assertEqual(diff['1-16 cu-in'], '1-16 cu-in (16-262 cubic centimeter)')
        self.assertEqual(diff['1-16 cb. in'], '1-16 cb. in (16-262 cubic centimeter)')
        self.assertEqual(diff['16.7-Cubic-in'], '16.7-Cubic-in (273.7 cubic centimeter)')
        self.assertEqual(diff['16,500-cu. in'], '16,500-cu. in (0.3 cubic meter)')
        self.assertEqual(res['body_html'], item['body_html'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.metadata.item import ITEM_STATE

states = []
actions = []
allowed_workflow_states = []

__all__ = ('workflow_state', 'get_workflow_states', 'allowed_workflow_states',
           'workflow_action', 'get_workflow_actions', 'is_workflow_state_transition_valid',
           'set_default_state')


def workflow_action(name, include_states=None, exclude_states=None, privileges=None):
    """Register new workflow action.

    :param name: unique name of action
    :param include_states: include action if item state is in include_states
    :param exclude_states: exclude action if item state is in exclude_statese
    :param privileges: list of privileges required for this action
    """
    if include_states is None:
        include_states = []
    if exclude_states is None:
        exclude_states = []
    if privileges is None:
        privileges = []
    actions.append({
        'name': name,
        'exclude_states': exclude_states,
        'include_states': include_states,
        'privileges': privileges
    })


def get_workflow_actions(state=None):
    """Get list of all registered workflow actions.

    :param state: filter actions by given item state if provided
    """
    if state is None:
        return actions
    else:
        def is_go(action, state):
            if action['include_states']:
                return state in action['include_states']
            elif action['exclude_states']:
                return state not in action['exclude_states']
        return [action for action in actions if is_go(action, state)]


def get_privileged_actions(privileges):
    """Get the actions that are within the privileged list

    :param privileges:
    :return: list of allowed actions
    """
    allowed_action = []
    for action in actions:
        # only add the action if it has some privileges
        add = len(action['privileges']) > 0
        for action_privilege in action['privileges']:
            if action_privilege not in privileges.keys():
                add = False
            else:
                # make sure the privilege is allowed
                if not privileges[action_privilege] == 1:
                    add = False
        if add:
            allowed_action.append(action)
    return allowed_action


def workflow_state(name):
    """Register new workflow state.

    :param name: unique name of state
    """
    allowed_workflow_states.append(name)
    states.append({'name': name})


def is_workflow_state_transition_valid(action_name, state):
    """
    Checks if the state transition is valid or not. It's assumed that there are no duplicate actions.

    :param action_name: Action Name as defined while registering Workflow Action.
    :param state: Current State of the content
    :return: true if valid, False otherwise.
    """
    return action_name in [action['name'] for action in get_workflow_actions(state)]


def get_workflow_states():
    """Get list of all registered workflow states."""
    return states


def set_default_state(doc, state):
    """Sets default state for given doc.

    :param doc: item
    :param state: state to be set as default
    """
    doc.setdefault(ITEM_STATE, state)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import lxml.etree as etree

from unittest import mock

from apps.publish import init_app
from superdesk.tests import TestCase
from superdesk.utc import utcnow
from superdesk.publish.formatters import NewsMLG2Formatter


NSMAP = {
    None: 'http://iptc.org/std/nar/2006-10-01/',
    'xml': 'http://www.w3.org/XML/1998/namespace',
}

XML_LANG = '{{{}}}lang'.format(NSMAP['xml'])


def ns(key):
    tokens = key.split('/')
    return '/'.join(['{http://iptc.org/std/nar/2006-10-01/}%s' % token for token in tokens])


@mock.patch('superdesk.publish.subscribers.SubscribersService.generate_sequence_number', lambda self, subscriber: 1)
class NewsMLG2FormatterTest(TestCase):
    embargo_ts = (utcnow() + datetime.timedelta(days=2))
    article = {
        'guid': 'tag:aap.com.au:20150613:12345',
        '_current_version': 1,
        'anpa_category': [
            {
                'qcode': 'a',
                'name': 'Australian General News'
            }
        ],
        'source': 'AAP',
        'headline': 'This is a test headline - для теста!',
        'byline': 'joe',
        'slugline': 'slugline',
        'subject': [{'qcode': '02011001', 'name': 'international court or tribunal'},
                    {'qcode': '02011002', 'name': 'extradition'}],
        'anpa_take_key': 'take_key',
        'unique_id': '1',
        'body_html': 'The story body',
        'type': 'text',
        'word_count': '1',
        'priority': '1',
        '_id': 'urn:localhost.abc',
        'state': 'published',
        'urgency': 2,
        'pubstatus': 'usable',
        'dateline': {
            'source': 'AAP',
            'text': 'Los Angeles, Aug 11 AAP -',
            'located': {
                'alt_name': '',
                'state': 'California',
                'city_code': 'Los Angeles',
                'city': 'Los Angeles',
                'dateline': 'city',
                'country_code': 'US',
                'country': 'USA',
                'tz': 'America/Los_Angeles',
                'state_code': 'CA'
            }
        },
        'creditline': 'sample creditline',
        'keywords': ['traffic'],
        'abstract': 'sample abstract',
        'place': [{'qcode': 'Australia', 'name': 'Australia',
                   'state': '', 'country': 'Australia',
                   'world_region': 'Oceania'}],
        'embargo': embargo_ts,
        'company_codes': [{'name': 'YANCOAL AUSTRALIA LIMITED', 'qcode': 'YAL', 'security_exchange': 'ASX'}],
        'genre': [
            {'name': 'Article (News)', 'qcode': 'article'},
            {'name': 'foo', 'code': 'foo', 'scheme': 'custom'},
        ],
    }

    package = {
        '_id': 'urn:newsml:localhost:2015-08-12T11:59:58.457029:7e90d257-92f6-406d-9186-95653b211701',
        'type': 'composite',
        '_current_version': 1,
        'groups': [
            {
                'role': 'grpRole:NEP',
                'id': 'root',
                'refs': [
                    {
                        'idRef': 'main'
                    },
                    {
                        'idRef': 'sidebar'
                    }
                ]
            },
            {
                'role': 'grpRole:main',
                'id': 'main',
                'refs': [
                    {
                        'type': 'text',
                        'renditions': {},
                        'itemClass': 'icls:text',
                        'guid': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'residRef': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'location': 'archive',
                        'headline': 'US:US cop sacked over student shooting',
                        'slugline': 'US Police'
                    }
                ]
            },
            {
                'id': 'sidebar',
                'refs': [
                    {
                        'type': 'text',
                        'itemClass': 'icls:text',
                        'headline': 'Sidebar',
                        'location': 'archive'
                    }
                ]
            }
        ],
        'pubstatus': 'usable',
        'state': 'published',
        'flags': {'marked_for_not_publication': False},
        'guid': 'urn:newsml:localhost:2015-08-12T11:59:58.457029:7e90d257-92f6-406d-9186-95653b211701',
        'dateline': {
            'located': {
                'alt_name': '',
                'state': 'California',
                'city_code': 'Los Angeles',
                'city': 'Los Angeles',
                'dateline': 'city',
                'country_code': 'US',
                'country': 'USA',
                'tz': 'America/Los_Angeles',
                'state_code': 'CA'
            },
            'date': '2015-08-12T01:59:58.000Z',
            'source': 'AAP',
            'text': 'Los Angeles, Aug 11 AAP -'
        },
        'language': 'en',
        'headline': 'Cop sacked over student shooting',
        'source': 'AAP',
        'slugline': 'US Police',
        'anpa_category': [
            {
                'name': 'International News',
                'qcode': 'I'
            }
        ],
        'subject': [
            {
                'name': 'police',
                'parent': '02000000',
                'qcode': '02003000'
            }
        ]
    }

    picture_package = {
        '_id': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'guid': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'language': 'en',
        'pubstatus': 'usable',
        'groups': [
            {
                'refs': [
                    {
                        'idRef': 'main'
                    }
                ],
                'id': 'root',
                'role': 'grpRole:NEP'
            },
            {
                'refs': [
                    {
                        'guid': '20150813001165688150',
                        'headline': 'Prison Riot',
                        'residRef': 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037',
                        'location': 'archive',
                        'type': 'picture',
                        'slugline': 'Prison Riot',
                        'renditions': {
                            'baseImage': {
                                'height': 1400,
                                'mimetype': 'image/jpeg',
                                'width': 1120,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650a'
                            },
                            'thumbnail': {
                                'height': 120,
                                'mimetype': 'image/jpeg',
                                'width': 96,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650c'
                            },
                            'viewImage': {
                                'height': 640,
                                'mimetype': 'image/jpeg',
                                'width': 512,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650e'
                            },
                            'original': {
                                'height': 800,
                                'mimetype': 'image/jpeg',
                                'width': 640,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b6508'
                            }
                        },
                        'itemClass': 'icls:picture'
                    }
                ],
                'id': 'main',
                'role': 'grpRole:main'
            }
        ],
        'type': 'composite',
        'state': 'published',
        'slugline': 'Prison Riot',
        'description': 'This Jan. 21, 2015 photo is of something)',
        'source': 'AAP',
        'headline': 'Prison Riot',
        '_current_version': 1,
        'dateline': {
            'date': '2015-08-13T04:07:59.000Z',
            'source': 'AAP'
        },
        'flags': {'marked_for_not_publication': False},
        'sign_off': 'mar',
    }

    picture = {
        '_id': 'tag:localhost:2015:cf15b059-b997-4e34-a103-85b8d7ea4ba3',
        'dateline': {
            'source': 'AAP',
            'date': '2015-08-14T04:45:53.000Z'
        },
        'renditions': {
            'viewImage': {
                'height': 415,
                'href': 'http://localhost:5000/api/upload/55cd72811d41c828e1773786/raw?_schema=http',
                'media': '55cd72811d41c828e1773786',
                'mimetype': 'image/jpeg',
                'width': 640
            },
            'baseImage': {
                'height': 909,
                'href': 'http://localhost:5000/api/upload/55cd72811d41c828e1773782/raw?_schema=http',
                'media': '55cd72811d41c828e1773782',
                'mimetype': 'image/jpeg',
                'width': 1400
            },
            'thumbnail': {
                'height': 120,
                'href': 'http://localhost:5000/api/upload/55cd72811d41c828e1773784/raw?_schema=http',
                'media': '55cd72811d41c828e1773784',
                'mimetype': 'image/jpeg',
                'width': 184
            },
            'original': {
                'height': 2455,
                'href': 'http://localhost:5000/api/upload/55cd72801d41c828e1773762/raw?_schema=http',
                'media': '55cd72801d41c828e1773762',
                'mimetype': 'image/jpeg',
                'width': 3777
            }
        },
        'state': 'published',
        'guid': '20150731001161435160',
        'source': 'AAP Image',
        '_current_version': 1,
        'original_source': 'AAP Image/AAP',
        'description': 'Federal Education Minister Christopher Pyne launches his new book NO ARCHIVING',
        'type': 'picture',
        'slugline': 'NUS CHRISTOPHER PYNE PROTEST',
        'headline': 'NUS CHRISTOPHER PYNE PROTEST',
        'pubstatus': 'usable',
        'ednote': '',
        'byline': 'TRACEY NEARMY',
        'filemeta': {
            'yresolution': [
                300,
                1
            ],
            'exposuretime': [
                1,
                200
            ],
            'copyright': '                                                      ',
            'scenecapturetype': 0,
            'sensingmethod': 2,
            'fnumber': [
                14,
                5
            ],
            'flashpixversion': '0100',
            'xresolution': [
                300,
                1
            ],
            'resolutionunit': 2,
            'subsectimedigitized': '20',
            'exposureprogram': 1,
            'subsectimeoriginal': '20',
            'make': 'NIKON CORPORATION',
            'focallengthin35mmfilm': 200,
            'scenetype': 1,
            'exifimageheight': 2455,
            'saturation': 0,
            'colorspace': 1,
            'subjectdistancerange': 0,
            'datetime': '2015:07:31 18:55:37',
            'software': 'Photogene for iPad v4.3',
            'flash': 16,
            'focallength': [
                200,
                1
            ],
            'componentsconfiguration': '\u0001\u0002\u0003\u0000',
            'lightsource': 3,
            'artist': '                                    ',
            'isospeedratings': 2000,
            'whitepoint': [
                313,
                1000
            ],
            'sharpness': 2,
            'exposuremode': 1,
            'meteringmode': 3,
            'compressedbitsperpixel': [
                4,
                1
            ],
            'model': 'NIKON D800E',
            'subsectime': '20',
            'datetimedigitized': '2015:07:31 18:55:37',
            'exifoffset': 406,
            'contrast': 0,
            'whitebalance': 1,
            'exifimagewidth': 3777,
            'datetimeoriginal': '2015:07:31 18:55:37',
            'customrendered': 0,
            'maxaperturevalue': [
                3,
                1
            ],
            'digitalzoomratio': [
                1,
                1
            ],
            'primarychromaticities': [
                16,
                25
            ],
            'length': 8009209,
            'exifversion': '0230',
            'gaincontrol': 2,
            'gamma': [
                11,
                5
            ],
            'filesource': 3
        },
        'language': 'en',
        'mimetype': 'image/jpeg',
        'sign_off': 'mar',
        'unique_id': 573
    }

    video = {
        '_id': 'urn:newsml:localhost:2015-09-20T16:12:57.333001:f3856812-0999-4ed8-b69e-68dcdeb1ed2e',
        'guid': 'tag:localhost:2015:c11e11c4-cdbc-41ef-b939-2b30dd8365fb',
        'language': 'en',
        'family_id': 'urn:newsml:localhost:2015-09-20T16:12:57.333001:f3856812-0999-4ed8-b69e-68dcdeb1ed2e',
        '_current_version': 3,
        'versioncreated': '2015-09-20T06:14:11.000Z',
        'unique_id': 274,
        'renditions': {
            'original': {
                'media': '55fe4e691d41c8cac923ceb2',
                'href': 'http://192.168.220.176:5000/api/upload/55fe4e691d41c8cac923ceb2/raw?_schema=http',
                'mimetype': 'video/mp4'
            }
        },
        'state': 'in_progress',
        'version_creator': '55ee82871d41c86ee1d78c45',
        'sign_off': 'ADM',
        'media': '55fe4e691d41c8cac923ceb2',
        'source': 'AAP',
        'original_source': 'AAP Video/AAP',
        'pubstatus': 'usable',
        'filemeta': {
            'mime_type': 'video/mp4',
            'last_modification': '1904-01-01T00:00:00+00:00',
            'creation_date': '1904-01-01T00:00:00+00:00',
            'height': '270',
            'width': '480',
            'duration': '0:00:10.224000',
            'comment': 'User volume: 100.0%',
            'length': 877869,
            'endian': 'Big endian'
        },
        'event_id': 'tag:localhost:2015:f3ae4441-4721-4987-8265-88d747b6a550',
        'original_creator': '55ee82871d41c86ee1d78c45',
        'expiry': '2016-12-21T14:14:11.000Z',
        'firstcreated': '2015-09-20T06:12:57.000Z',
        '_created': '2015-09-20T06:12:57.000Z',
        'type': 'video',
        'unique_name': '#274',
        'mimetype': 'video/mp4',
        'version': 2,
        'headline': 'test',
        'description': 'testing video',
        'abstract': 'test video',
        'slugline': 'test video',
        'byline': 'test video',
        'subject': [
            {
                'qcode': '01001000',
                'name': 'archaeology',
                'parent': '01000000'
            }
        ],
        'place': [
            {
                'qcode': 'ACT',
                'name': 'ACT'
            }
        ],
        'anpa_category': [
            {
                'qcode': 'a',
                'name': 'Australian General News'
            }
        ]
    }

    picture_text_package = {
        '_id': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'guid': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'language': 'en',
        'pubstatus': 'usable',
        'groups': [
            {
                'refs': [
                    {
                        'idRef': 'main'
                    }
                ],
                'id': 'root',
                'role': 'grpRole:NEP'
            },
            {
                'refs': [
                    {
                        'type': 'text',
                        'renditions': {},
                        'itemClass': 'icls:text',
                        'guid': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'residRef': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'location': 'archive',
                        'headline': 'US:US cop sacked over student shooting',
                        'slugline': 'US Police',
                        '_current_version': 4
                    },
                    {
                        'guid': '20150813001165688150',
                        'headline': 'Prison Riot',
                        'residRef': 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037',
                        'location': 'archive',
                        'type': 'picture',
                        'slugline': 'Prison Riot',
                        'renditions': {
                            'baseImage': {
                                'height': 1400,
                                'mimetype': 'image/jpeg',
                                'width': 1120,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650a'
                            },
                            'thumbnail': {
                                'height': 120,
                                'mimetype': 'image/jpeg',
                                'width': 96,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650c'
                            },
                            'viewImage': {
                                'height': 640,
                                'mimetype': 'image/jpeg',
                                'width': 512,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650e'
                            },
                            'original': {
                                'height': 800,
                                'mimetype': 'image/jpeg',
                                'width': 640,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b6508'
                            }
                        },
                        'itemClass': 'icls:picture',
                        '_current_version': 4
                    }
                ],
                'id': 'main',
                'role': 'grpRole:main'
            }
        ],
        'type': 'composite',
        'state': 'published',
        'slugline': 'Prison Riot',
        'description': 'This Jan. 21, 2015 photo is of something)',
        'source': 'AAP',
        'headline': 'Prison Riot',
        '_current_version': 1,
        'dateline': {
            'date': '2015-08-13T04:07:59.000Z',
            'source': 'AAP'
        },
        'flags': {'marked_for_not_publication': False},
        'sign_off': 'mar',
    }

    picture_text_package_multi_group = {
        '_id': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'guid': 'urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e',
        'language': 'en',
        'pubstatus': 'usable',
        'groups': [
            {
                'refs': [
                    {'idRef': 'main'}, {'idRef': 'picture'}
                ],
                'id': 'root',
                'role': 'grpRole:NEP'
            },
            {
                'refs': [
                    {
                        'type': 'text',
                        'renditions': {},
                        'itemClass': 'icls:text',
                        'guid': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'residRef': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                        'location': 'archive',
                        'headline': 'US:US cop sacked over student shooting',
                        'slugline': 'US Police',
                        '_current_version': 4
                    }
                ],
                'id': 'main',
                'role': 'grpRole:main'
            },
            {
                'refs': [
                    {
                        'guid': '20150813001165688150',
                        'headline': 'Prison Riot',
                        'residRef': 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037',
                        'location': 'archive',
                        'type': 'picture',
                        'slugline': 'Prison Riot',
                        'renditions': {
                            'baseImage': {
                                'height': 1400,
                                'mimetype': 'image/jpeg',
                                'width': 1120,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650a'
                            },
                            'thumbnail': {
                                'height': 120,
                                'mimetype': 'image/jpeg',
                                'width': 96,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650c'
                            },
                            'viewImage': {
                                'height': 640,
                                'mimetype': 'image/jpeg',
                                'width': 512,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b650e'
                            },
                            'original': {
                                'height': 800,
                                'mimetype': 'image/jpeg',
                                'width': 640,
                                'href': 'http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http',
                                'media': '55cc03731d41c8cea12b6508'
                            }
                        },
                        'itemClass': 'icls:picture',
                        '_current_version': 4
                    }
                ],
                'id': 'picture',
                'role': 'grpRole:picture'
            }
        ],
        'type': 'composite',
        'state': 'published',
        'slugline': 'Prison Riot',
        'description': 'This Jan. 21, 2015 photo is of something)',
        'source': 'AAP',
        'headline': 'Prison Riot',
        '_current_version': 1,
        'dateline': {
            'date': '2015-08-13T04:07:59.000Z',
            'source': 'AAP'
        },
        'flags': {'marked_for_not_publication': False},
        'sign_off': 'mar',
    }

    packaged_articles = [{'_id': 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b',
                          'headline': 'package article headline',
                          'slugline': 'slugline',
                          'pubStatus': 'usable'},
                         {'_id': 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037',
                          'headline': 'package article headline',
                          'slugline': 'slugline',
                          'pubStatus': 'usable'}]

    vocab = [{'_id': 'rightsinfo', 'items': [{'name': 'AAP',
                                              'copyrightHolder': 'copy right holder',
                                              'copyrightNotice': 'copy right notice',
                                              'usageTerms': 'terms'},
                                             {'name': 'default',
                                              'copyrightHolder': 'default copy right holder',
                                              'copyrightNotice': 'default copy right notice',
                                              'usageTerms': 'default terms'}]}]

    now = datetime.datetime(2015, 6, 13, 11, 45, 19, 0)

    def setUp(self):
        self.article['state'] = 'published'
        self.article['firstcreated'] = self.now
        self.article['versioncreated'] = self.now
        self.newsml = etree.Element("NewsML")
        self.formatter = NewsMLG2Formatter()
        self.formatter.now = self.now
        self.formatter.string_now = self.now.strftime('%Y-%m-%dT%H:%M:%S.0000Z')
        init_app(self.app)
        self.app.data.insert('vocabularies', self.vocab)
        self.app.data.insert('archive', self.packaged_articles)

    def test_formatter(self):
        seq, doc = self.formatter.format(self.article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        content_meta = xml.find(ns('itemSet')).find(ns('newsItem')).find(ns('contentMeta'))
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}sender').text,
            'sourcefabric.org')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority').text,
            '1')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}origin').text, 'AAP')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}rightsInfo/{http://iptc.org/std/nar/2006-10-01/}usageTerms').text,
            'terms')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}itemMeta/{http://iptc.org/std/nar/2006-10-01/}provider/' +
            '{http://iptc.org/std/nar/2006-10-01/}name').text,
            'sourcefabric.org')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/{http://iptc.org/std/nar/2006-10-01/}headline').text,
            'This is a test headline - для теста!')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/{http://iptc.org/std/nar/2006-10-01/}urgency').text,
            '2')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/{http://iptc.org/std/nar/2006-10-01/}dateline').text,
            'Los Angeles, Aug 11 AAP -')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/' +
            '{http://iptc.org/std/nar/2006-10-01/}located[@qcode="loctyp:City"]/' +
            '{http://iptc.org/std/nar/2006-10-01/}name').text, 'Los Angeles')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/' +
            '{http://iptc.org/std/nar/2006-10-01/}located/' +
            '{http://iptc.org/std/nar/2006-10-01/}broader[@qcode="loctyp:CountryArea"]/' +
            '{http://iptc.org/std/nar/2006-10-01/}name').text, 'California')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/' +
            '{http://iptc.org/std/nar/2006-10-01/}located/' +
            '{http://iptc.org/std/nar/2006-10-01/}broader[@qcode="loctyp:Country"]/' +
            '{http://iptc.org/std/nar/2006-10-01/}name').text, 'USA')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/{http://iptc.org/std/nar/2006-10-01/}inlineXML/' +
            '{http://iptc.org/std/nar/2006-10-01/}nitf/{http://iptc.org/std/nar/2006-10-01/}body/' +
            '{http://iptc.org/std/nar/2006-10-01/}body.content/' +
            '{http://iptc.org/std/nar/2006-10-01/}p').text, 'The story body')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}itemMeta/{http://iptc.org/std/nar/2006-10-01/}embargoed').text,
            self.embargo_ts.isoformat())

        company_details = xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}assert/{http://iptc.org/std/nar/2006-10-01/}organisationDetails/' +
            '{http://iptc.org/std/nar/2006-10-01/}hasInstrument[@symbol]')
        self.assertEqual(company_details.attrib.get('symbol', ''), 'YAL')
        self.assertEqual(company_details.attrib.get('marketlabel', ''), 'ASX')

        company_details = xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/' +
            '{http://iptc.org/std/nar/2006-10-01/}subject[@type="cpnat:organisation"]/' +
            '{http://iptc.org/std/nar/2006-10-01/}name')
        self.assertEqual(company_details.text, 'YANCOAL AUSTRALIA LIMITED')

        genre = content_meta.findall(ns('genre'))
        self.assertEqual(2, len(genre))
        self.assertEqual('article', genre[0].get('qcode'))
        self.assertEqual('Article (News)', genre[0].find(ns('name')).text)
        self.assertEqual('custom:foo', genre[1].get('qcode'))

    def testPreservedFomat(self):
        article = dict(self.article)
        article['format'] = 'preserved'
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/{http://iptc.org/std/nar/2006-10-01/}inlineData').text,
            'The story body')

    def testDefaultRightsFomatter(self):
        article = dict(self.article)
        article['source'] = 'BOGUS'
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}rightsInfo/{http://iptc.org/std/nar/2006-10-01/}usageTerms').text,
            'default terms')

    def testPackagePublish(self):
        article = dict(self.package)
        article['firstcreated'] = self.now
        article['versioncreated'] = self.now
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority').text,
            '5')

        groups = xml.findall(ns('itemSet/packageItem/groupSet/group'))
        self.assertEqual(3, len(groups))

        self.assertEqual(xml.find(ns('itemSet/packageItem/groupSet/group/itemRef/slugline')).text, 'slugline')
        self.assertEqual(xml.find(ns('itemSet/packageItem/groupSet/group/itemRef')).get('residref'),
                         'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b')

    def testPicturePackagePublish(self):
        article = dict(self.picture_package)
        article['firstcreated'] = self.now
        article['versioncreated'] = self.now
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority').text,
            '5')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}packageItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}groupSet/{http://iptc.org/std/nar/2006-10-01/}group/' +
            '{http://iptc.org/std/nar/2006-10-01/}itemRef/{http://iptc.org/std/nar/2006-10-01/}slugline').text,
            'slugline')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}packageItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}groupSet/{http://iptc.org/std/nar/2006-10-01/}group/' +
            '{http://iptc.org/std/nar/2006-10-01/}itemRef').get('residref'),
            'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037')

    def testPicturePublish(self):
        article = dict(self.picture)
        article['firstcreated'] = self.now
        article['versioncreated'] = self.now
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority').text,
            '5')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/' +
            '{http://iptc.org/std/nar/2006-10-01/}creditline').text,
            'AAP Image/AAP')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/' +
            '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]').get('width'),
            '3777')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/' +
            '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]').get('height'),
            '2455')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/' +
            '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]').get('contenttype'),
            'image/jpeg')

    def testVideoPublish(self):
        article = dict(self.video)
        article['firstcreated'] = self.now
        article['versioncreated'] = self.now
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority').text,
            '5')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentMeta/' +
            '{http://iptc.org/std/nar/2006-10-01/}creditline').text,
            'AAP Video/AAP')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/' +
            '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]').get('width'),
            '480')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/' +
            '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]').get('height'),
            '270')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/' +
            '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]').get('duration'),
            '0:00:10.224000')
        self.assertEqual(xml.find(
            '{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/' +
            '{http://iptc.org/std/nar/2006-10-01/}contentSet/' +
            '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]').get('contenttype'),
            'video/mp4')

    def testPictureTextPackage(self):
        article = dict(self.picture_text_package)
        article['firstcreated'] = self.now
        article['versioncreated'] = self.now
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        item_refs = xml.findall('.//{http://iptc.org/std/nar/2006-10-01/}itemRef')
        self.assertEqual(len(item_refs), 2)
        self.assertEqual(item_refs[0].get('residref'), 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b')
        self.assertEqual(item_refs[0].find('{http://iptc.org/std/nar/2006-10-01/}itemClass').get('qcode'),
                         'ninat:text')
        self.assertEqual(item_refs[0].find('{http://iptc.org/std/nar/2006-10-01/}pubStatus').get('qcode'),
                         'stat:usable')
        self.assertEqual(item_refs[1].get('residref'), 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037')
        self.assertEqual(item_refs[1].find('{http://iptc.org/std/nar/2006-10-01/}itemClass').get('qcode'),
                         'ninat:picture')
        self.assertEqual(item_refs[1].find('{http://iptc.org/std/nar/2006-10-01/}pubStatus').get('qcode'),
                         'stat:usable')

    def testPictureTextPackageMultiGroup(self):
        article = dict(self.picture_text_package_multi_group)
        article['firstcreated'] = self.now
        article['versioncreated'] = self.now
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        item_refs = xml.findall('.//{http://iptc.org/std/nar/2006-10-01/}itemRef')
        self.assertEqual(len(item_refs), 2)
        self.assertEqual(item_refs[0].get('residref'), 'tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b')
        self.assertEqual(item_refs[0].find('{http://iptc.org/std/nar/2006-10-01/}itemClass').get('qcode'),
                         'ninat:text')
        self.assertEqual(item_refs[0].find('{http://iptc.org/std/nar/2006-10-01/}pubStatus').get('qcode'),
                         'stat:usable')
        self.assertEqual(item_refs[1].get('residref'), 'tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037')
        self.assertEqual(item_refs[1].find('{http://iptc.org/std/nar/2006-10-01/}itemClass').get('qcode'),
                         'ninat:picture')
        self.assertEqual(item_refs[1].find('{http://iptc.org/std/nar/2006-10-01/}pubStatus').get('qcode'),
                         'stat:usable')
        groups = xml.findall('.//{http://iptc.org/std/nar/2006-10-01/}group')
        self.assertEqual(len(groups), 3)
        self.assertEqual(groups[0].get('id'), 'root')
        self.assertEqual(groups[0].get('role'), 'grpRole:NEP')
        self.assertEqual(groups[1].get('id'), 'main')
        self.assertEqual(groups[1].get('role'), 'grpRole:main')
        self.assertEqual(groups[2].get('id'), 'picture')
        self.assertEqual(groups[2].get('role'), 'grpRole:picture')

        group_ref = xml.findall('.//{http://iptc.org/std/nar/2006-10-01/}groupRef')
        self.assertEqual(len(group_ref), 2)
        self.assertEqual(group_ref[0].get('idref'), 'main')
        self.assertEqual(group_ref[1].get('idref'), 'picture')

    def testPlace(self):
        article = self.article.copy()
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        content_meta = xml.find('{http://iptc.org/std/nar/2006-10-01/}itemSet'
                                '/{http://iptc.org/std/nar/2006-10-01/}newsItem/'
                                '{http://iptc.org/std/nar/2006-10-01/}contentMeta')
        subject = content_meta.find('{http://iptc.org/std/nar/2006-10-01/}subject[@qcode="loctyp:Country"]')
        self.assertEqual(subject.find('{http://iptc.org/std/nar/2006-10-01/}name').text, "Australia")
        self.assertEqual(subject.find('{http://iptc.org/std/nar/2006-10-01/}broader' +
                                      '[@qcode="loctyp:WorldArea"]/' +
                                      '{http://iptc.org/std/nar/2006-10-01/}name').text, "Oceania")
        self.assertIsNone(content_meta.find('{http://iptc.org/std/nar/2006-10-01/}'
                                            'subject[@qcode="loctyp:CountryArea"]'))
        article['place'] = [{"name": "ACT", "qcode": "ACT",
                             "state": "Australian Capital Territory",
                             "country": "Australia", "world_region": "Oceania"}]
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        content_meta = xml.find('{http://iptc.org/std/nar/2006-10-01/}itemSet'
                                '/{http://iptc.org/std/nar/2006-10-01/}newsItem/'
                                '{http://iptc.org/std/nar/2006-10-01/}contentMeta')
        subject = content_meta.find('{http://iptc.org/std/nar/2006-10-01/}subject[@qcode="loctyp:CountryArea"]')
        self.assertEqual(subject.find('{http://iptc.org/std/nar/2006-10-01/}name').text,
                         "Australian Capital Territory")
        self.assertEqual(subject.find('{http://iptc.org/std/nar/2006-10-01/}broader' +
                                      '[@qcode="loctyp:Country"]/' +
                                      '{http://iptc.org/std/nar/2006-10-01/}name').text, "Australia")
        self.assertEqual(subject.find('{http://iptc.org/std/nar/2006-10-01/}broader' +
                                      '[@qcode="loctyp:WorldArea"]/' +
                                      '{http://iptc.org/std/nar/2006-10-01/}name').text, "Oceania")

        article['place'] = [{"name": "EUR", "qcode": "EUR",
                             "state": "", "country": "", "world_region": "Europe"}]
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        content_meta = xml.find('{http://iptc.org/std/nar/2006-10-01/}itemSet'
                                '/{http://iptc.org/std/nar/2006-10-01/}newsItem/'
                                '{http://iptc.org/std/nar/2006-10-01/}contentMeta')
        subject = content_meta.find('{http://iptc.org/std/nar/2006-10-01/}subject[@qcode="loctyp:WorldArea"]')
        self.assertEqual(subject.find('{http://iptc.org/std/nar/2006-10-01/}name').text, "Europe")
        self.assertIsNone(content_meta.find('{http://iptc.org/std/nar/2006-10-01/}'
                                            'subject[@qcode="loctyp:CountryArea"]'))
        self.assertIsNone(content_meta.find('{http://iptc.org/std/nar/2006-10-01/}'
                                            'subject[@qcode="loctyp:Country"]'))

    def testPlaceGeonames(self):
        article = self.article.copy()
        article['place'] = [
            {
                "name": "Kobeřice",
                "code": "3073493",
                "scheme": "geonames",
                "state": "Moravskoslezský",
                "country": "Czechia",
                "state_code": "80",
                "country_code": "CZ",
                "feature_class": "P",
                "location": {
                    "lat": 49.98548,
                    "lon": 18.05212,
                },
            },
            {
                "name": "Moravskoslezsky",
                "code": "3339573",
                "scheme": "geonames",
                "state": "Moravskoslezský",
                "country": "Czechia",
                "state_code": "80",
                "country_code": "CZ",
                "feature_class": "A",
                "location": {
                    "lat": 49.83333,
                    "lon": 18.0,
                },
            },
            {
                "name": "Czechia",
                "code": "3077311",
                "scheme": "geonames",
                "state": "",
                "state_code": None,
                "country": "Czechia",
                "country_code": "CZ",
                "feature_class": "A",
                "location": {
                    "lat": 49.75,
                    "lon": 15.0,
                },
            },
        ]

        _, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        xml = etree.fromstring(doc.encode('utf-8'))
        content_meta = xml.find(ns('itemSet')).find(ns('newsItem')).find(ns('contentMeta'))

        # test city
        subject = content_meta.find(ns('subject[@type="cpnat:geoArea"]'))
        self.assertIsNotNone(subject, etree.tostring(content_meta, pretty_print=True))
        self.assertEqual(article['place'][0]['name'], subject.find(ns('name')).text)

        broader = subject.findall(ns('broader'))
        self.assertEqual(2, len(broader))
        self.assertEqual('iso3166-2:CZ-80', broader[0].get('qcode'))
        self.assertEqual('cptype:statprov', broader[0].get('type'))
        self.assertEqual(article['place'][0]['state'], broader[0].find(ns('name')).text)
        self.assertEqual('iso3166-1a2:CZ', broader[1].get('qcode'))
        self.assertEqual('cptype:country', broader[1].get('type'))
        self.assertEqual(article['place'][0]['country'], broader[1].find(ns('name')).text)

        geo_area = subject.find(ns('geoAreaDetails'))
        self.assertIsNotNone(geo_area)
        position = geo_area.find(ns('position'))
        self.assertIsNotNone(position)
        self.assertEqual('49.98548', position.get('latitude'))
        self.assertEqual('18.05212', position.get('longitude'))

        subjects = content_meta.findall(ns('subject[@type="cpnat:geoArea"]'))
        self.assertEqual(3, len(subjects))

        # test state
        state = subjects[1]
        self.assertEqual('Moravskoslezsky', state.find(ns('name')).text)
        self.assertEqual('iso3166-2:CZ-80', state.get('qcode'))
        broader = state.findall(ns('broader'))
        self.assertEqual(1, len(broader))
        self.assertEqual('iso3166-1a2:CZ', broader[0].get('qcode'))
        self.assertEqual('cptype:country', broader[0].get('type'))
        self.assertEqual('Czechia', broader[0].find(ns('name')).text)

        # test country
        country = subjects[2]
        self.assertEqual('Czechia', country.find(ns('name')).text)
        self.assertEqual('iso3166-1a2:CZ', country.get('qcode'))
        broader = country.findall(ns('broader'))
        self.assertEqual(0, len(broader))

    def format(self, updates=None):
        article = self.article.copy()
        article.update(updates)
        _, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        root = etree.fromstring(doc.encode('utf-8'))
        item = root.find('itemSet', NSMAP).find('newsItem', NSMAP)
        return item

    def test_lang_fr(self):
        item = self.format({
            'language': 'fr-CA',
            'subject': [
                {'name': 'foo', 'qcode': 'foo', 'translations': {
                    'name': {'fr': 'foo fr', 'en': 'foo en'},
                }},
                {'name': 'bar', 'qcode': 'bar'},
            ],
            'anpa_category': [
                {'name': 'cat', 'qcode': 'cat', 'translations': {
                    'name': {'fr-CA': 'cat fr', 'en': 'cat en'},
                }},
            ],
            'place': [],
            'company_codes': [],
            'genre': [
                {'name': 'gen', 'qcode': 'gen', 'translations': {
                    'name': {'fr': 'gen fr', 'en': 'gen en'},
                }},
            ],
        })

        self.assertIsNotNone(item)
        self.assertEqual('fr-CA', item.attrib.get(XML_LANG))

        meta = item.find('contentMeta', NSMAP)
        self.assertIsNotNone(meta)

        subject = meta.findall('subject', NSMAP)
        self.assertEqual(3, len(subject))
        self.assertEqual('foo fr', subject[0].find('name', NSMAP).text)
        self.assertEqual('fr', subject[0].find('name', NSMAP).attrib[XML_LANG])
        self.assertEqual('bar', subject[1].find('name', NSMAP).text)
        self.assertEqual('en', subject[1].find('name', NSMAP).attrib[XML_LANG])
        self.assertEqual('cat fr', subject[2].find('name', NSMAP).text)
        self.assertEqual('fr-CA', subject[2].find('name', NSMAP).attrib[XML_LANG])

        genre = meta.find('genre', NSMAP)
        self.assertEqual('gen fr', genre.find('name', NSMAP).text)
        self.assertEqual('fr', genre.find('name', NSMAP).attrib[XML_LANG])




from superdesk import get_resource_service
from superdesk.utc import utcnow
from superdesk.logging import logger
from flask_babel import lazy_gettext


def get_destination_desk(desk, limit=10):
    if not limit or not desk:
        return
    if not desk.get('is_closed'):
        return desk
    if not desk.get('closed_destination'):
        return desk
    return get_destination_desk(
        get_resource_service('desks').find_one(req=None, _id=desk['closed_destination']),
        limit - 1
    )


def routing(item, desk=None, **kwargs):
    if desk is None:
        desk_id = item.get('task', {}).get('desk')
        if desk_id:
            desk = get_resource_service('desks').find_one(req=None, _id=desk_id)
    dest = get_destination_desk(desk)
    if dest and str(desk['_id']) != str(dest['_id']):
        logger.info('auto-routing item "%s" from desk "%s" to "%s"',
                    item.get('headline'), desk.get('name'), dest.get('name'))
        try:
            marked_desks = item.get('marked_desks', [])
            existing = [mark for mark in marked_desks if str(mark['desk_id']) == str(dest['_id'])]
            if not existing:
                marked_desks.append({
                    'desk_id': str(dest['_id']),
                    'date_marked': utcnow(),
                })
                item['marked_desks'] = marked_desks
        except Exception:
            logger.exception('auto-routing error')
    return item


name = 'desk_routing'
label = lazy_gettext('Desk Routing')
callback = routing
access_type = 'backend'
action_type = 'direct'
